/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.testsuites;

import java.awt.*;

import org.gfxtest.framework.*;
import org.gfxtest.framework.annotations.*;



/**
 * These tests renders various lines using identity transformation matrix.
 * 
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.RENDER_TEST)
@GraphicsPrimitive(GraphicsPrimitives.LINE)
@RenderStyle(RenderStyles.NORMAL)
@Transformation(Transformations.NONE)
@Zoom(1)
public class NormalLines extends GfxTest
{

    /**
     * Width of border where lines are not drawn
     */
    private static final int BORDER_WIDTH = 30;

    /**
     * Step used for drawing sequence of horizontal and/or vertical lines
     */
    private static final int STEP = 20;

    /**
     * Test basic behavior of method Graphics.drawLine().
     *
     * @param image
     *            image to which line is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawLine(TestImage image, Graphics2D graphics)
    {
        // set drawing color
        graphics.setColor(Color.BLACK);

        // draw two diagonals along the whole test image
        graphics.drawLine(BORDER_WIDTH, BORDER_WIDTH, image.getWidth() - BORDER_WIDTH, image.getHeight() - BORDER_WIDTH);
        graphics.drawLine(BORDER_WIDTH, image.getHeight() - BORDER_WIDTH, image.getWidth() - BORDER_WIDTH, BORDER_WIDTH);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawLine(). Here vertical lines
     * are drawn using various colors.
     * 
     * @param image
     *            image to which line is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawLineVert1(TestImage image, Graphics2D graphics)
    {
        int colorIndex = 0;
        for (int x = BORDER_WIDTH; x < image.getWidth() - BORDER_WIDTH; x += STEP)
        {
            // set drawing color
            graphics.setColor(ColorPalette.getColor(colorIndex++));
            // draw vertical line
            graphics.drawLine(x, BORDER_WIDTH, x, image.getHeight() - BORDER_WIDTH);
        }

        // test return value
        return TestResult.PASSED;
    }
    
    /**
     * Test basic behavior of method Graphics.drawLine(). Here vertical lines
     * are drawn using one color but with various stroke width using default caps.
     * 
     * @param image
     *            image to which line is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawLineVert2(TestImage image, Graphics2D graphics)
    {
        // set drawing color
        graphics.setColor(Color.BLACK);
        float strokeWidth = 0;
        for (int x = BORDER_WIDTH; x < image.getWidth() - BORDER_WIDTH; x += STEP)
        {
            // set stroke width
            graphics.setStroke(new BasicStroke(strokeWidth));
            // increase stroke width
            strokeWidth += 0.5;
            // draw vertical line
            graphics.drawLine(x, BORDER_WIDTH, x, image.getHeight() - BORDER_WIDTH);
        }

        // test return value
        return TestResult.PASSED;
    }
    
    /**
     * Test basic behavior of method Graphics.drawLine(). Here vertical lines
     * are drawn using one color but with various stroke width using CAP_BUTT
     * to render both line end points.
     * 
     * @param image
     *            image to which line is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawLineVert3(TestImage image, Graphics2D graphics)
    {
        // set drawing color
        graphics.setColor(Color.BLACK);
        float strokeWidth = 0;
        for (int x = BORDER_WIDTH; x < image.getWidth() - BORDER_WIDTH; x += STEP)
        {
            // set stroke width
            graphics.setStroke(new BasicStroke(strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
            // increase stroke width
            strokeWidth += 0.5;
            // draw vertical line
            graphics.drawLine(x, BORDER_WIDTH, x, image.getHeight() - BORDER_WIDTH);
        }

        // test return value
        return TestResult.PASSED;
    }
    
    /**
     * Test basic behavior of method Graphics.drawLine(). Here vertical lines
     * are drawn using one color but with various stroke width using CAP_ROUND
     * to render both line end points.
     *
     * @param image
     *            image to which line is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawLineVert4(TestImage image, Graphics2D graphics)
    {
        // set drawing color
        graphics.setColor(Color.BLACK);
        float strokeWidth = 0;
        for (int x = BORDER_WIDTH; x < image.getWidth() - BORDER_WIDTH; x += STEP)
        {
            // set stroke width
            graphics.setStroke(new BasicStroke(strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL));
            // increase stroke width
            strokeWidth += 0.5;
            // draw vertical line
            graphics.drawLine(x, BORDER_WIDTH, x, image.getHeight() - BORDER_WIDTH);
        }

        // test return value
        return TestResult.PASSED;
    }
    
    
    /**
     * Test basic behavior of method Graphics.drawLine(). Here vertical lines
     * are drawn using one color but with various stroke width using CAP_SQUARE
     * to render both line end points.
     * 
     * @param image
     *            image to which line is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawLineVert5(TestImage image, Graphics2D graphics)
    {
        // set drawing color
        graphics.setColor(Color.BLACK);
        float strokeWidth = 0;
        for (int x = BORDER_WIDTH; x < image.getWidth() - BORDER_WIDTH; x += STEP)
        {
            // set stroke width
            graphics.setStroke(new BasicStroke(strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL));
            // increase stroke width
            strokeWidth += 0.5;
            // draw vertical line
            graphics.drawLine(x, BORDER_WIDTH, x, image.getHeight() - BORDER_WIDTH);
        }

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawLine(). Here horizontal lines
     * are drawn using various colors.
     * 
     * @param image
     *            image to which line is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawLineHors1(TestImage image, Graphics2D graphics)
    {
        int colorIndex = 0;
        for (int y = BORDER_WIDTH; y < image.getWidth() - BORDER_WIDTH; y += STEP)
        {
            // set drawing color
            graphics.setColor(ColorPalette.getColor(colorIndex++));
            // draw horizontal line
            graphics.drawLine(BORDER_WIDTH, y, image.getWidth() - BORDER_WIDTH, y);
        }

        // test return value
        return TestResult.PASSED;
    }
    
    /**
     * Test basic behavior of method Graphics.drawLine(). Here horizontal lines
     * are drawn using one color but with various stroke width using default caps.
     * 
     * @param image
     *            image to which line is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawLineHors2(TestImage image, Graphics2D graphics)
    {
        // set drawing color
        graphics.setColor(Color.BLACK);
        float strokeWidth = 0;
        for (int y = BORDER_WIDTH; y < image.getWidth() - BORDER_WIDTH; y += STEP)
        {
            // set stroke width
            graphics.setStroke(new BasicStroke(strokeWidth));
            // increase stroke width
            strokeWidth += 0.5;
            // draw horizontal line
            graphics.drawLine(BORDER_WIDTH, y, image.getWidth() - BORDER_WIDTH, y);        }

        // test return value
        return TestResult.PASSED;
    }
    
    /**
     * Test basic behavior of method Graphics.drawLine(). Here horizontal lines
     * are drawn using one color but with various stroke width using CAP_BUTT
     * to render both line end points.
     * 
     * @param image
     *            image to which line is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawLineHors3(TestImage image, Graphics2D graphics)
    {
        // set drawing color
        graphics.setColor(Color.BLACK);
        float strokeWidth = 0;
        for (int y = BORDER_WIDTH; y < image.getWidth() - BORDER_WIDTH; y += STEP)
        {
            // set stroke width
            graphics.setStroke(new BasicStroke(strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
            // increase stroke width
            strokeWidth += 0.5;
            // draw horizontal line
            graphics.drawLine(BORDER_WIDTH, y, image.getWidth() - BORDER_WIDTH, y);
        }

        // test return value
        return TestResult.PASSED;
    }
    
    /**
     * Test basic behavior of method Graphics.drawLine(). Here horizontal lines
     * are drawn using one color but with various stroke width using CAP_ROUND
     * to render both line end points.
     *
     * @param image
     *            image to which line is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawLineHors4(TestImage image, Graphics2D graphics)
    {
        // set drawing color
        graphics.setColor(Color.BLACK);
        float strokeWidth = 0;
        for (int y = BORDER_WIDTH; y < image.getWidth() - BORDER_WIDTH; y += STEP)
        {
            // set stroke width
            graphics.setStroke(new BasicStroke(strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL));
            // increase stroke width
            strokeWidth += 0.5;
            // draw horizontal line
            graphics.drawLine(BORDER_WIDTH, y, image.getWidth() - BORDER_WIDTH, y);
        }

        // test return value
        return TestResult.PASSED;
    }
    
    
    /**
     * Test basic behavior of method Graphics.drawLine(). Here horizontal lines
     * are drawn using one color but with various stroke width using CAP_SQUARE
     * to render both line end points.
     * 
     * @param image
     *            image to which line is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawLineHors5(TestImage image, Graphics2D graphics)
    {
        // set drawing color
        graphics.setColor(Color.BLACK);
        float strokeWidth = 0;
        for (int y = BORDER_WIDTH; y < image.getWidth() - BORDER_WIDTH; y += STEP)
        {
            // set stroke width
            graphics.setStroke(new BasicStroke(strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL));
            // increase stroke width
            strokeWidth += 0.5;
            // draw horizontal line
            graphics.drawLine(BORDER_WIDTH, y, image.getWidth() - BORDER_WIDTH, y);
        }

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Draw line entity using various rendering styles. This method is
     * automatically called by test harness.
     * 
     * @param image
     *            image to which line is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @param testNumber
     *            test number set by test harness
     * @param entityRenderingStyle
     *            entity rendering style set by test harness
     */
    @Override
    protected void drawEntity(TestImage image, Graphics2D graphics, int testNumber, EntityRenderingStyle entityRenderingStyle)
    {
        // set entity rendering style
        graphics.setStroke(new BasicStroke(entityRenderingStyle.getWidth(), entityRenderingStyle.getCap(), entityRenderingStyle.getJoin()));
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        for (int i = 0; i < ANGLES; i++)
        {
            Color color = new Color(Color.HSBtoRGB((float)i/ANGLES, 1.0f, 1.0f));
            graphics.setColor(color);
            int majorRadius = (xc > yc ? yc : xc) - 20;
            double angle = 2.0 * i / ANGLES * Math.PI;
            double cos = Math.cos(angle);
            double sin = Math.sin(angle);
            int x1 = xc + (int)(MINOR_RADIUS*cos);
            int y1 = yc + (int)(MINOR_RADIUS*sin);
            int x2 = xc + (int)(majorRadius*cos);
            int y2 = yc + (int)(majorRadius*sin);
            graphics.drawLine(x1, y1, x2, y2);
        }
        // draw cross to mark center of the test image
        drawCross(graphics, xc, yc, CROSS_SIZE);
    }

    /**
     * Method which is automatically called by test harness.
     * 
     * @param configuration
     *            current graphics test configuration
     */
    @Override
    protected void runOtherTests(GfxTestConfiguration configuration)
    {
        drawEntityWithVariousStyles(configuration, true, false, true, false, false);
    }

    /**
     * Entry point to the test suite.
     *
     * @param args not used in this case
     */
    public static void main(String[] args)
    {
        new NormalLines().runTestSuite(args);
    }
}
